﻿unit UTemplateReplacer;

interface

uses
  Winapi.Windows, System.SysUtils, System.Classes, System.IOUtils, System.Math,
  Vcl.Menus, ToolsAPI;

const
  TemplateCount = 9; // XE4-10.2 Tokyo
  TemplateNames: array[0..TemplateCount-1] of string = (
    'New Data Module',
    'New Unit',
    'New Component',
    'New Thread',
    'New Named Thread',
    'New Library',
    'New Package',
    'New Console Project',
    'New Project'
  );

type
  TModuleSources = array[0..TemplateCount-1] of string;
  PModuleSources = ^TModuleSources;

implementation

uses
  UTemplateReplacerForm;

const
  DelphiCoreIdeName = {$IFDEF VER250}'delphicoreide180.bpl'{$ENDIF} // XE4
                      {$IFDEF VER260}'delphicoreide190.bpl'{$ENDIF} // XE5
                      {$IFDEF VER270}'delphicoreide200.bpl'{$ENDIF} // XE6
                      {$IFDEF VER280}'delphicoreide210.bpl'{$ENDIF} // XE7
                      {$IFDEF VER290}'delphicoreide220.bpl'{$ENDIF} // XE8
                      {$IFDEF VER300}'delphicoreide230.bpl'{$ENDIF} // 10 Seattle
                      {$IFDEF VER310}'delphicoreide240.bpl'{$ENDIF} // 10.1 Berlin
                      {$IFDEF VER320}'delphicoreide250.bpl'{$ENDIF} // 10.2 Tokyo
                      {$IFDEF VER330}'delphicoreide260.bpl'{$ENDIF} // 10.3 Rio
                      ;
  PasCrtrsModuleSourcesName = '@Pascrtrs@ModuleSources';

var
  FDefaultModuleSources: TModuleSources;
  FUserDefinedModuleSources: TModuleSources;
  FIDEMenu: TMenuItem;

procedure LoadTemplates;
var
  path: string;
  arr: TArray<string>;
  i: Integer;
begin
  path := ExtractFilePath(GetModuleName(HInstance)) + 'ModuleSources.txt';
  if not FileExists(path) then Exit;

  arr := TFile.ReadAllText(path, TEncoding.UTF8).Split(['|']);
  if Length(arr) <> TemplateCount then Exit;

  for i := Low(TModuleSources) to High(TModuleSources) do
    FUserDefinedModuleSources[i] := arr[i];
end;

procedure SaveTemplates;
var
  s: string;
  i: Integer;
begin
  s := FUserDefinedModuleSources[0];
  for i := Low(TModuleSources)+1 to High(TModuleSources) do
    s := s + '|' + FUserDefinedModuleSources[i];
  TFile.WriteAllText(ExtractFilePath(GetModuleName(HInstance)) + 'ModuleSources.txt', s, TEncoding.UTF8);
end;

procedure ReplaceTemplates(const ASources: TModuleSources);
var
  sources: PModuleSources;
begin
  sources := GetProcAddress(GetModuleHandle(DelphiCoreIdeName), PasCrtrsModuleSourcesName);
  if sources = nil then Exit;

  sources^ := ASources;
end;

procedure InitTemplates;
var
  sources: PModuleSources;
begin
  sources := GetProcAddress(GetModuleHandle(DelphiCoreIdeName), PasCrtrsModuleSourcesName);
  if sources = nil then Exit;

  FDefaultModuleSources := sources^;
  FUserDefinedModuleSources := sources^;

  LoadTemplates;
  ReplaceTemplates(FUserDefinedModuleSources);
end;

procedure IDEMenuClick(Self, Sender: TObject);
begin
  if not TTemplateReplacerForm.Execute(FUserDefinedModuleSources, FDefaultModuleSources) then Exit;

  ReplaceTemplates(FUserDefinedModuleSources);
  SaveTemplates;
end;

procedure RegisterMenu;
var
  nta: INTAServices;
  i, j: Integer;
  method: TMethod;
begin
  nta := BorlandIDEServices as INTAServices;
  for i := 0 to nta.MainMenu.Items.Count-1 do
  begin
    if nta.MainMenu.Items[i].Name <> 'ToolsMenu' then Continue;

    for j := 0 to nta.MainMenu.Items[i].Count-1 do
    begin
      if nta.MainMenu.Items[i][j].Name <> 'ToolsToolsItem' then Continue;

      FIDEMenu := TMenuItem.Create(nil);
      FIDEMenu.AutoHotkeys := maManual;
      FIDEMenu.Caption := 'Template Replacer';
      method.Code := @IDEMenuClick;
      method.Data := nil;
      FIDEMenu.OnClick := TNotifyEvent(method);
      nta.MainMenu.Items[i].Insert(j, FIDEMenu);
      Break;
    end;
    Break;
  end;
end;

procedure UnregisterMenu;
begin
  FIDEMenu.Free;
end;

initialization
  InitTemplates;
  RegisterMenu;
finalization
  ReplaceTemplates(FDefaultModuleSources);
  UnregisterMenu;
end.
